package fr.inria.phoenix.diasuite.framework.context.efficiency;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.device.heatingmeter.CurrentHeatingConsumptionFromHeatingMeter;

/**
 * Compute instant efficiency for each heating measurement
 * 
 * <pre>
 * context Efficiency as EfficiencySample {
 * 	when provided currentHeatingConsumption from HeatingMeter
 * 		get targetTemperature from Thermostat,
 * 			temperature from TemperatureSensor
 * 		always publish;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractEfficiency extends Service {
    
    public AbstractEfficiency(ServiceConfiguration serviceConfiguration) {
        super("/Context/Efficiency/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        postInitialize();
    }
    
    @Override
    protected void postInitialize() {
        // Default implementation of post initialize: subscribe to all required devices
        discoverHeatingMeterForSubscribe.all().subscribeCurrentHeatingConsumption(); // subscribe to currentHeatingConsumption from all HeatingMeter devices
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("currentHeatingConsumption") && source.isCompatible("/Device/Device/PhysicalDevice/Sensor/EnergyMeter/HeatingMeter/")) {
            CurrentHeatingConsumptionFromHeatingMeter currentHeatingConsumptionFromHeatingMeter = new CurrentHeatingConsumptionFromHeatingMeter(this, source, (fr.inria.phoenix.diasuite.framework.datatype.state.State) value);
            
            setEfficiency(onCurrentHeatingConsumptionFromHeatingMeter(currentHeatingConsumptionFromHeatingMeter, new DiscoverForCurrentHeatingConsumptionFromHeatingMeter()));
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("efficiency")) {
            return getLastValue();
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private fr.inria.phoenix.diasuite.framework.datatype.efficiencysample.EfficiencySample contextValue;
    
    private void setEfficiency(fr.inria.phoenix.diasuite.framework.datatype.efficiencysample.EfficiencySample newContextValue) {
        contextValue = newContextValue;
        getProcessor().publishValue(getOutProperties(), "efficiency", newContextValue);
    }
    
    /**
     * Get the last value of the context
     * 
     * @return the latest value published by the context
     */
    protected final fr.inria.phoenix.diasuite.framework.datatype.efficiencysample.EfficiencySample getLastValue() {
        return contextValue;
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when a <code>HeatingMeter</code> device on which we have subscribed publish on its <code>currentHeatingConsumption</code> source.
     * 
     * <pre>
     * when provided currentHeatingConsumption from HeatingMeter
     * 		get targetTemperature from Thermostat,
     * 			temperature from TemperatureSensor
     * 		always publish;
     * </pre>
     * 
     * @param currentHeatingConsumptionFromHeatingMeter the value of the <code>currentHeatingConsumption</code> source and the <code>HeatingMeter</code> device that published the value.
     * @param discover a discover object to get value from devices and contexts
     * @return the value to publish
     */
    protected abstract fr.inria.phoenix.diasuite.framework.datatype.efficiencysample.EfficiencySample onCurrentHeatingConsumptionFromHeatingMeter(CurrentHeatingConsumptionFromHeatingMeter currentHeatingConsumptionFromHeatingMeter, DiscoverForCurrentHeatingConsumptionFromHeatingMeter discover);
    
    // End of interaction contract implementation
    
    // Discover part for HeatingMeter devices
    /**
     * Use this object to discover HeatingMeter devices.
     * 
     * <pre>
     * device HeatingMeter extends EnergyMeter {
     * 	source currentHeatingConsumption as State;
     * }
     * </pre>
     * 
     * @see HeatingMeterDiscoverer
     */
    protected final HeatingMeterDiscoverer discoverHeatingMeterForSubscribe = new HeatingMeterDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>HeatingMeter</code> devices that can be discovered
     * 
     * <pre>
     * device HeatingMeter extends EnergyMeter {
     * 	source currentHeatingConsumption as State;
     * }
     * </pre>
     */
    protected final static class HeatingMeterDiscoverer {
        private Service serviceParent;
        
        private HeatingMeterDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private HeatingMeterComposite instantiateComposite() {
            return new HeatingMeterComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>HeatingMeter</code> devices
         * 
         * @return a {@link HeatingMeterComposite} object composed of all discoverable <code>HeatingMeter</code>
         */
        public HeatingMeterComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>HeatingMeter</code> devices
         * 
         * @return a {@link HeatingMeterProxy} object pointing to a random discoverable <code>HeatingMeter</code> device
         */
        public HeatingMeterProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>HeatingMeter</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link HeatingMeterComposite} object composed of all matching <code>HeatingMeter</code> devices
         */
        public HeatingMeterComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>HeatingMeter</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link HeatingMeterComposite} object composed of all matching <code>HeatingMeter</code> devices
         */
        public HeatingMeterComposite whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>HeatingMeter</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link HeatingMeterComposite} object composed of all matching <code>HeatingMeter</code> devices
         */
        public HeatingMeterComposite whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>HeatingMeter</code> devices
     * 
     * <pre>
     * device HeatingMeter extends EnergyMeter {
     * 	source currentHeatingConsumption as State;
     * }
     * </pre>
     */
    protected final static class HeatingMeterComposite extends fr.inria.diagen.core.service.composite.Composite<HeatingMeterProxy> {
        private HeatingMeterComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Sensor/EnergyMeter/HeatingMeter/");
        }
        
        @Override
        protected HeatingMeterProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new HeatingMeterProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link HeatingMeterComposite}, filtered using the attribute <code>id</code>.
         */
        public HeatingMeterComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link HeatingMeterComposite}, filtered using the attribute <code>location</code>.
         */
        public HeatingMeterComposite andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link HeatingMeterComposite}, filtered using the attribute <code>user</code>.
         */
        public HeatingMeterComposite andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Subscribes to the <code>currentHeatingConsumption</code> source. After a call to this method, the context will be notified when a
         * <code>HeatingMeter</code> device of this composite publishes a value on its <code>currentHeatingConsumption</code> source.
         */
        public void subscribeCurrentHeatingConsumption() {
            subscribeValue("currentHeatingConsumption");
        }
        
        /**
         * Unsubscribes from the <code>currentHeatingConsumption</code> source. After a call to this method, the context will no more be notified
         * when a <code>HeatingMeter</code> device of this composite publishes a value on its <code>currentHeatingConsumption</code> source.
         */
        public void unsubscribeCurrentHeatingConsumption() {
            unsubscribeValue("currentHeatingConsumption");
        }
    }
    
    /**
     * A proxy to one <code>HeatingMeter</code> device
     * 
     * <pre>
     * device HeatingMeter extends EnergyMeter {
     * 	source currentHeatingConsumption as State;
     * }
     * </pre>
     */
    protected final static class HeatingMeterProxy extends Proxy {
        private HeatingMeterProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>currentHeatingConsumption</code> source. After a call to this method, the context will be notified when the
         * <code>HeatingMeter</code> device of this proxy publishes a value on its <code>currentHeatingConsumption</code> source.
         */
        public void subscribeCurrentHeatingConsumption() {
            subscribeValue("currentHeatingConsumption");
        }
        
        /**
         * Unsubscribes from the <code>currentHeatingConsumption</code> source. After a call to this method, the context will no more be notified
         * when the <code>HeatingMeter</code> device of this proxy publishes a value on its <code>currentHeatingConsumption</code> source.
         */
        public void unsubscribeCurrentHeatingConsumption() {
            unsubscribeValue("currentHeatingConsumption");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover part for HeatingMeter devices
    
    // Discover object for currentHeatingConsumption from HeatingMeter
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided currentHeatingConsumption from HeatingMeter
     * 		get targetTemperature from Thermostat,
     * 			temperature from TemperatureSensor
     * 		always publish;
     * </code>
     */
    protected final class DiscoverForCurrentHeatingConsumptionFromHeatingMeter {
        private final ThermostatDiscovererForCurrentHeatingConsumptionFromHeatingMeter thermostatDiscoverer = new ThermostatDiscovererForCurrentHeatingConsumptionFromHeatingMeter(AbstractEfficiency.this);
        private final TemperatureSensorDiscovererForCurrentHeatingConsumptionFromHeatingMeter temperatureSensorDiscoverer = new TemperatureSensorDiscovererForCurrentHeatingConsumptionFromHeatingMeter(AbstractEfficiency.this);
        
        /**
         * @return a {@link ThermostatDiscovererForCurrentHeatingConsumptionFromHeatingMeter} object to discover <code>Thermostat</code> devices
         */
        public ThermostatDiscovererForCurrentHeatingConsumptionFromHeatingMeter thermostats() {
            return thermostatDiscoverer;
        }
        
        /**
         * @return a {@link TemperatureSensorDiscovererForCurrentHeatingConsumptionFromHeatingMeter} object to discover <code>TemperatureSensor</code> devices
         */
        public TemperatureSensorDiscovererForCurrentHeatingConsumptionFromHeatingMeter temperatureSensors() {
            return temperatureSensorDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>Thermostat</code> devices to get their sources for the
     * <code>when provided currentHeatingConsumption from HeatingMeter</code> interaction contract.
     * 
     * <pre>
     * device Thermostat extends Appliance {
     * 	action SetTemperature;
     * 	source targetTemperature as Temperature;
     * }
     * </pre>
     */
    protected final static class ThermostatDiscovererForCurrentHeatingConsumptionFromHeatingMeter {
        private Service serviceParent;
        
        private ThermostatDiscovererForCurrentHeatingConsumptionFromHeatingMeter(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter instantiateComposite() {
            return new ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices
         * 
         * @return a {@link ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter} object composed of all discoverable <code>Thermostat</code>
         */
        public ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Thermostat</code> devices
         * 
         * @return a {@link ThermostatProxyForCurrentHeatingConsumptionFromHeatingMeter} object pointing to a random discoverable <code>Thermostat</code> device
         */
        public ThermostatProxyForCurrentHeatingConsumptionFromHeatingMeter anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter} object composed of all matching <code>Thermostat</code> devices
         */
        public ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter} object composed of all matching <code>Thermostat</code> devices
         */
        public ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter} object composed of all matching <code>Thermostat</code> devices
         */
        public ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>Thermostat</code> devices to get their sources for the
     * <code>when provided currentHeatingConsumption from HeatingMeter</code> interaction contract.
     * 
     * <pre>
     * device Thermostat extends Appliance {
     * 	action SetTemperature;
     * 	source targetTemperature as Temperature;
     * }
     * </pre>
     */
    protected final static class ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter extends fr.inria.diagen.core.service.composite.Composite<ThermostatProxyForCurrentHeatingConsumptionFromHeatingMeter> {
        private ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Appliance/Thermostat/");
        }
        
        @Override
        protected ThermostatProxyForCurrentHeatingConsumptionFromHeatingMeter instantiateProxy(RemoteServiceInfo rsi) {
            return new ThermostatProxyForCurrentHeatingConsumptionFromHeatingMeter(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter}, filtered using the attribute <code>id</code>.
         */
        public ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter}, filtered using the attribute <code>location</code>.
         */
        public ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter}, filtered using the attribute <code>user</code>.
         */
        public ThermostatCompositeForCurrentHeatingConsumptionFromHeatingMeter andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
    }
    
    /**
     * A proxy to one <code>Thermostat</code> device to get its sources for the
     * <code>when provided currentHeatingConsumption from HeatingMeter</code> interaction contract.
     * 
     * <pre>
     * device Thermostat extends Appliance {
     * 	action SetTemperature;
     * 	source targetTemperature as Temperature;
     * }
     * </pre>
     */
    protected final static class ThermostatProxyForCurrentHeatingConsumptionFromHeatingMeter extends Proxy {
        private ThermostatProxyForCurrentHeatingConsumptionFromHeatingMeter(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Returns the value of the <code>targetTemperature</code> source of this <code>Thermostat</code> device
         * 
         * @return the value of the <code>targetTemperature</code> source
         */
        public fr.inria.phoenix.diasuite.framework.datatype.temperature.Temperature getTargetTemperature() throws InvocationException {
            return (fr.inria.phoenix.diasuite.framework.datatype.temperature.Temperature) callGetValue("targetTemperature");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    
    /**
     * Discover object that will exposes the <code>TemperatureSensor</code> devices to get their sources for the
     * <code>when provided currentHeatingConsumption from HeatingMeter</code> interaction contract.
     * 
     * <pre>
     * device TemperatureSensor extends Sensor {
     * 	source temperature as Temperature;
     * }
     * </pre>
     */
    protected final static class TemperatureSensorDiscovererForCurrentHeatingConsumptionFromHeatingMeter {
        private Service serviceParent;
        
        private TemperatureSensorDiscovererForCurrentHeatingConsumptionFromHeatingMeter(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter instantiateComposite() {
            return new TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>TemperatureSensor</code> devices
         * 
         * @return a {@link TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter} object composed of all discoverable <code>TemperatureSensor</code>
         */
        public TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>TemperatureSensor</code> devices
         * 
         * @return a {@link TemperatureSensorProxyForCurrentHeatingConsumptionFromHeatingMeter} object pointing to a random discoverable <code>TemperatureSensor</code> device
         */
        public TemperatureSensorProxyForCurrentHeatingConsumptionFromHeatingMeter anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>TemperatureSensor</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter} object composed of all matching <code>TemperatureSensor</code> devices
         */
        public TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>TemperatureSensor</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter} object composed of all matching <code>TemperatureSensor</code> devices
         */
        public TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>TemperatureSensor</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter} object composed of all matching <code>TemperatureSensor</code> devices
         */
        public TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>TemperatureSensor</code> devices to get their sources for the
     * <code>when provided currentHeatingConsumption from HeatingMeter</code> interaction contract.
     * 
     * <pre>
     * device TemperatureSensor extends Sensor {
     * 	source temperature as Temperature;
     * }
     * </pre>
     */
    protected final static class TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter extends fr.inria.diagen.core.service.composite.Composite<TemperatureSensorProxyForCurrentHeatingConsumptionFromHeatingMeter> {
        private TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Sensor/TemperatureSensor/");
        }
        
        @Override
        protected TemperatureSensorProxyForCurrentHeatingConsumptionFromHeatingMeter instantiateProxy(RemoteServiceInfo rsi) {
            return new TemperatureSensorProxyForCurrentHeatingConsumptionFromHeatingMeter(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter}, filtered using the attribute <code>id</code>.
         */
        public TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter}, filtered using the attribute <code>location</code>.
         */
        public TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter}, filtered using the attribute <code>user</code>.
         */
        public TemperatureSensorCompositeForCurrentHeatingConsumptionFromHeatingMeter andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
    }
    
    /**
     * A proxy to one <code>TemperatureSensor</code> device to get its sources for the
     * <code>when provided currentHeatingConsumption from HeatingMeter</code> interaction contract.
     * 
     * <pre>
     * device TemperatureSensor extends Sensor {
     * 	source temperature as Temperature;
     * }
     * </pre>
     */
    protected final static class TemperatureSensorProxyForCurrentHeatingConsumptionFromHeatingMeter extends Proxy {
        private TemperatureSensorProxyForCurrentHeatingConsumptionFromHeatingMeter(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Returns the value of the <code>temperature</code> source of this <code>TemperatureSensor</code> device
         * 
         * @return the value of the <code>temperature</code> source
         */
        public fr.inria.phoenix.diasuite.framework.datatype.temperature.Temperature getTemperature() throws InvocationException {
            return (fr.inria.phoenix.diasuite.framework.datatype.temperature.Temperature) callGetValue("temperature");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover object for currentHeatingConsumption from HeatingMeter
}
