package fr.inria.phoenix.diasuite.framework.context.format;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.device.heatingmeter.CurrentHeatingConsumptionFromHeatingMeter;

/**
 * * HeatingMonitor contexts
 * Format various events for logging purpose
 * 
 * <pre>
 * context Format as Event {
 * 	when provided currentHeatingConsumption from HeatingMeter
 * 		always publish;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractFormat extends Service {
    
    public AbstractFormat(ServiceConfiguration serviceConfiguration) {
        super("/Context/Format/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        postInitialize();
    }
    
    @Override
    protected void postInitialize() {
        // Default implementation of post initialize: subscribe to all required devices
        discoverHeatingMeterForSubscribe.all().subscribeCurrentHeatingConsumption(); // subscribe to currentHeatingConsumption from all HeatingMeter devices
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("currentHeatingConsumption") && source.isCompatible("/Device/Device/PhysicalDevice/Sensor/EnergyMeter/HeatingMeter/")) {
            CurrentHeatingConsumptionFromHeatingMeter currentHeatingConsumptionFromHeatingMeter = new CurrentHeatingConsumptionFromHeatingMeter(this, source, (fr.inria.phoenix.diasuite.framework.datatype.state.State) value);
            
            setFormat(onCurrentHeatingConsumptionFromHeatingMeter(currentHeatingConsumptionFromHeatingMeter));
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("format")) {
            return getLastValue();
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private fr.inria.phoenix.diasuite.framework.datatype.event.Event contextValue;
    
    private void setFormat(fr.inria.phoenix.diasuite.framework.datatype.event.Event newContextValue) {
        contextValue = newContextValue;
        getProcessor().publishValue(getOutProperties(), "format", newContextValue);
    }
    
    /**
     * Get the last value of the context
     * 
     * @return the latest value published by the context
     */
    protected final fr.inria.phoenix.diasuite.framework.datatype.event.Event getLastValue() {
        return contextValue;
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when a <code>HeatingMeter</code> device on which we have subscribed publish on its <code>currentHeatingConsumption</code> source.
     * 
     * <pre>
     * when provided currentHeatingConsumption from HeatingMeter
     * 		always publish;
     * </pre>
     * 
     * @param currentHeatingConsumptionFromHeatingMeter the value of the <code>currentHeatingConsumption</code> source and the <code>HeatingMeter</code> device that published the value.
     * @return the value to publish
     */
    protected abstract fr.inria.phoenix.diasuite.framework.datatype.event.Event onCurrentHeatingConsumptionFromHeatingMeter(CurrentHeatingConsumptionFromHeatingMeter currentHeatingConsumptionFromHeatingMeter);
    
    // End of interaction contract implementation
    
    // Discover part for HeatingMeter devices
    /**
     * Use this object to discover HeatingMeter devices.
     * 
     * <pre>
     * device HeatingMeter extends EnergyMeter {
     * 	source currentHeatingConsumption as State;
     * }
     * </pre>
     * 
     * @see HeatingMeterDiscoverer
     */
    protected final HeatingMeterDiscoverer discoverHeatingMeterForSubscribe = new HeatingMeterDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>HeatingMeter</code> devices that can be discovered
     * 
     * <pre>
     * device HeatingMeter extends EnergyMeter {
     * 	source currentHeatingConsumption as State;
     * }
     * </pre>
     */
    protected final static class HeatingMeterDiscoverer {
        private Service serviceParent;
        
        private HeatingMeterDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private HeatingMeterComposite instantiateComposite() {
            return new HeatingMeterComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>HeatingMeter</code> devices
         * 
         * @return a {@link HeatingMeterComposite} object composed of all discoverable <code>HeatingMeter</code>
         */
        public HeatingMeterComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>HeatingMeter</code> devices
         * 
         * @return a {@link HeatingMeterProxy} object pointing to a random discoverable <code>HeatingMeter</code> device
         */
        public HeatingMeterProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>HeatingMeter</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link HeatingMeterComposite} object composed of all matching <code>HeatingMeter</code> devices
         */
        public HeatingMeterComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>HeatingMeter</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link HeatingMeterComposite} object composed of all matching <code>HeatingMeter</code> devices
         */
        public HeatingMeterComposite whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>HeatingMeter</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link HeatingMeterComposite} object composed of all matching <code>HeatingMeter</code> devices
         */
        public HeatingMeterComposite whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>HeatingMeter</code> devices
     * 
     * <pre>
     * device HeatingMeter extends EnergyMeter {
     * 	source currentHeatingConsumption as State;
     * }
     * </pre>
     */
    protected final static class HeatingMeterComposite extends fr.inria.diagen.core.service.composite.Composite<HeatingMeterProxy> {
        private HeatingMeterComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Sensor/EnergyMeter/HeatingMeter/");
        }
        
        @Override
        protected HeatingMeterProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new HeatingMeterProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link HeatingMeterComposite}, filtered using the attribute <code>id</code>.
         */
        public HeatingMeterComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link HeatingMeterComposite}, filtered using the attribute <code>location</code>.
         */
        public HeatingMeterComposite andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link HeatingMeterComposite}, filtered using the attribute <code>user</code>.
         */
        public HeatingMeterComposite andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Subscribes to the <code>currentHeatingConsumption</code> source. After a call to this method, the context will be notified when a
         * <code>HeatingMeter</code> device of this composite publishes a value on its <code>currentHeatingConsumption</code> source.
         */
        public void subscribeCurrentHeatingConsumption() {
            subscribeValue("currentHeatingConsumption");
        }
        
        /**
         * Unsubscribes from the <code>currentHeatingConsumption</code> source. After a call to this method, the context will no more be notified
         * when a <code>HeatingMeter</code> device of this composite publishes a value on its <code>currentHeatingConsumption</code> source.
         */
        public void unsubscribeCurrentHeatingConsumption() {
            unsubscribeValue("currentHeatingConsumption");
        }
    }
    
    /**
     * A proxy to one <code>HeatingMeter</code> device
     * 
     * <pre>
     * device HeatingMeter extends EnergyMeter {
     * 	source currentHeatingConsumption as State;
     * }
     * </pre>
     */
    protected final static class HeatingMeterProxy extends Proxy {
        private HeatingMeterProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>currentHeatingConsumption</code> source. After a call to this method, the context will be notified when the
         * <code>HeatingMeter</code> device of this proxy publishes a value on its <code>currentHeatingConsumption</code> source.
         */
        public void subscribeCurrentHeatingConsumption() {
            subscribeValue("currentHeatingConsumption");
        }
        
        /**
         * Unsubscribes from the <code>currentHeatingConsumption</code> source. After a call to this method, the context will no more be notified
         * when the <code>HeatingMeter</code> device of this proxy publishes a value on its <code>currentHeatingConsumption</code> source.
         */
        public void unsubscribeCurrentHeatingConsumption() {
            unsubscribeValue("currentHeatingConsumption");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover part for HeatingMeter devices
}
