package fr.inria.phoenix.diasuite.framework.context.window;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.device.contactsensor.StateFromContactSensor;

/**
 * Cut/restart heating when window is open/closed
 * 
 * <pre>
 * context Window as Boolean indexed by user as String {
 * 	when provided state from ContactSensor
 * 	get status from Thermostat maybe publish;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractWindow extends Service {
    
    public AbstractWindow(ServiceConfiguration serviceConfiguration) {
        super("/Context/Window/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        postInitialize();
    }
    
    @Override
    protected void postInitialize() {
        // Default implementation of post initialize: subscribe to all required devices
        discoverContactSensorForSubscribe.all().subscribeState(); // subscribe to state from all ContactSensor devices
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("state") && source.isCompatible("/Device/Device/PhysicalDevice/Sensor/ContactSensor/")) {
            StateFromContactSensor stateFromContactSensor = new StateFromContactSensor(this, source, (fr.inria.phoenix.diasuite.framework.datatype.state.State) value);
            
            WindowValuePublishable returnValue = onStateFromContactSensor(stateFromContactSensor, new DiscoverForStateFromContactSensor());
            if(returnValue != null && returnValue.doPublish()) {
                setWindow(returnValue.getValue(), returnValue.getUser());
            }
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("window")) {
            return getLastValue((java.lang.String) indexes[0]);
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private java.util.Map<WindowIndices, java.lang.Boolean> contextValues = new java.util.HashMap<WindowIndices, java.lang.Boolean>();
    
    private void setWindow(java.lang.Boolean newContextValue, java.lang.String user) {
        WindowIndices indices = new WindowIndices(user);
        contextValues.put(indices, newContextValue);
        getProcessor().publishValue(getOutProperties(), "window", newContextValue, user);
    }
    
    /**
     * Get the last value of the context
     * 
     * @param user the index <code>user</code> for the last value
     * @return the latest value published by the context for the given indexes
     */
    protected final java.lang.Boolean getLastValue(java.lang.String user) {
        WindowIndices indices = new WindowIndices(user);
        return contextValues.get(indices);
    }
    
    /**
     * A class that represents a value that might be published for the <code>Window</code> context. It is used by
     * event methods that might or might not publish values for this context.
     */
    protected final static class WindowValuePublishable {
        
        // The value of the context
        private java.lang.Boolean value;
        // The user index
        private java.lang.String user;
        // Whether the value should be published or not
        private boolean doPublish;
        
        public WindowValuePublishable(java.lang.Boolean value,
                java.lang.String user,
                boolean doPublish) {
            this.value = value;
            this.user = user;
            this.doPublish = doPublish;
        }
        
        /**
         * @return the value of the context that might be published
         */
        public java.lang.Boolean getValue() {
            return value;
        }
        
        /**
         * Sets the value that might be published
         * 
         * @param value the value that will be published if {@link #doPublish()} returns true
         */
        public void setValue(java.lang.Boolean value) {
            this.value = value;
        }
        
        /**
         * @return the <code>user</code> index for the value of the context that might be published
         */
        public java.lang.String getUser() {
            return user;
        }
        
        /**
         * Sets the <code>user</code> index for the value that might be published
         * 
         * @param user the <code>user</code> index
         */
        public void setUser(java.lang.String user) {
            this.user = user;
        }
        
        /**
         * @return true if the value should be published
         */
        public boolean doPublish() {
            return doPublish;
        }
        
        /**
         * Set the value to be publishable or not
         * 
         * @param doPublish if true, the value will be published
         */
        public void setDoPublish(boolean doPublish) {
            this.doPublish = doPublish;
        }
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when a <code>ContactSensor</code> device on which we have subscribed publish on its <code>state</code> source.
     * 
     * <pre>
     * when provided state from ContactSensor
     * 	get status from Thermostat maybe publish;
     * </pre>
     * 
     * @param stateFromContactSensor the value of the <code>state</code> source and the <code>ContactSensor</code> device that published the value.
     * @param discover a discover object to get value from devices and contexts
     * @return a {@link WindowValuePublishable} that says if the context should publish a value and which value it should publish
     */
    protected abstract WindowValuePublishable onStateFromContactSensor(StateFromContactSensor stateFromContactSensor, DiscoverForStateFromContactSensor discover);
    
    // End of interaction contract implementation
    
    // Discover part for ContactSensor devices
    /**
     * Use this object to discover ContactSensor devices.
     * 
     * <pre>
     * device ContactSensor extends Sensor {
     * 	source state as State;
     * }
     * </pre>
     * 
     * @see ContactSensorDiscoverer
     */
    protected final ContactSensorDiscoverer discoverContactSensorForSubscribe = new ContactSensorDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>ContactSensor</code> devices that can be discovered
     * 
     * <pre>
     * device ContactSensor extends Sensor {
     * 	source state as State;
     * }
     * </pre>
     */
    protected final static class ContactSensorDiscoverer {
        private Service serviceParent;
        
        private ContactSensorDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private ContactSensorComposite instantiateComposite() {
            return new ContactSensorComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>ContactSensor</code> devices
         * 
         * @return a {@link ContactSensorComposite} object composed of all discoverable <code>ContactSensor</code>
         */
        public ContactSensorComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>ContactSensor</code> devices
         * 
         * @return a {@link ContactSensorProxy} object pointing to a random discoverable <code>ContactSensor</code> device
         */
        public ContactSensorProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>ContactSensor</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link ContactSensorComposite} object composed of all matching <code>ContactSensor</code> devices
         */
        public ContactSensorComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>ContactSensor</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link ContactSensorComposite} object composed of all matching <code>ContactSensor</code> devices
         */
        public ContactSensorComposite whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>ContactSensor</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link ContactSensorComposite} object composed of all matching <code>ContactSensor</code> devices
         */
        public ContactSensorComposite whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>ContactSensor</code> devices
     * 
     * <pre>
     * device ContactSensor extends Sensor {
     * 	source state as State;
     * }
     * </pre>
     */
    protected final static class ContactSensorComposite extends fr.inria.diagen.core.service.composite.Composite<ContactSensorProxy> {
        private ContactSensorComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Sensor/ContactSensor/");
        }
        
        @Override
        protected ContactSensorProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new ContactSensorProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link ContactSensorComposite}, filtered using the attribute <code>id</code>.
         */
        public ContactSensorComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link ContactSensorComposite}, filtered using the attribute <code>location</code>.
         */
        public ContactSensorComposite andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link ContactSensorComposite}, filtered using the attribute <code>user</code>.
         */
        public ContactSensorComposite andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Subscribes to the <code>state</code> source. After a call to this method, the context will be notified when a
         * <code>ContactSensor</code> device of this composite publishes a value on its <code>state</code> source.
         */
        public void subscribeState() {
            subscribeValue("state");
        }
        
        /**
         * Unsubscribes from the <code>state</code> source. After a call to this method, the context will no more be notified
         * when a <code>ContactSensor</code> device of this composite publishes a value on its <code>state</code> source.
         */
        public void unsubscribeState() {
            unsubscribeValue("state");
        }
    }
    
    /**
     * A proxy to one <code>ContactSensor</code> device
     * 
     * <pre>
     * device ContactSensor extends Sensor {
     * 	source state as State;
     * }
     * </pre>
     */
    protected final static class ContactSensorProxy extends Proxy {
        private ContactSensorProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>state</code> source. After a call to this method, the context will be notified when the
         * <code>ContactSensor</code> device of this proxy publishes a value on its <code>state</code> source.
         */
        public void subscribeState() {
            subscribeValue("state");
        }
        
        /**
         * Unsubscribes from the <code>state</code> source. After a call to this method, the context will no more be notified
         * when the <code>ContactSensor</code> device of this proxy publishes a value on its <code>state</code> source.
         */
        public void unsubscribeState() {
            unsubscribeValue("state");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover part for ContactSensor devices
    
    // Discover object for state from ContactSensor
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided state from ContactSensor
     * 	get status from Thermostat maybe publish;
     * </code>
     */
    protected final class DiscoverForStateFromContactSensor {
        private final ThermostatDiscovererForStateFromContactSensor thermostatDiscoverer = new ThermostatDiscovererForStateFromContactSensor(AbstractWindow.this);
        
        /**
         * @return a {@link ThermostatDiscovererForStateFromContactSensor} object to discover <code>Thermostat</code> devices
         */
        public ThermostatDiscovererForStateFromContactSensor thermostats() {
            return thermostatDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>Thermostat</code> devices to get their sources for the
     * <code>when provided state from ContactSensor</code> interaction contract.
     * 
     * <pre>
     * device Thermostat extends Appliance {
     * 	action SetTemperature;
     * 	source targetTemperature as Temperature;
     * }
     * </pre>
     */
    protected final static class ThermostatDiscovererForStateFromContactSensor {
        private Service serviceParent;
        
        private ThermostatDiscovererForStateFromContactSensor(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private ThermostatCompositeForStateFromContactSensor instantiateComposite() {
            return new ThermostatCompositeForStateFromContactSensor(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices
         * 
         * @return a {@link ThermostatCompositeForStateFromContactSensor} object composed of all discoverable <code>Thermostat</code>
         */
        public ThermostatCompositeForStateFromContactSensor all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Thermostat</code> devices
         * 
         * @return a {@link ThermostatProxyForStateFromContactSensor} object pointing to a random discoverable <code>Thermostat</code> device
         */
        public ThermostatProxyForStateFromContactSensor anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link ThermostatCompositeForStateFromContactSensor} object composed of all matching <code>Thermostat</code> devices
         */
        public ThermostatCompositeForStateFromContactSensor whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link ThermostatCompositeForStateFromContactSensor} object composed of all matching <code>Thermostat</code> devices
         */
        public ThermostatCompositeForStateFromContactSensor whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link ThermostatCompositeForStateFromContactSensor} object composed of all matching <code>Thermostat</code> devices
         */
        public ThermostatCompositeForStateFromContactSensor whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>Thermostat</code> devices to get their sources for the
     * <code>when provided state from ContactSensor</code> interaction contract.
     * 
     * <pre>
     * device Thermostat extends Appliance {
     * 	action SetTemperature;
     * 	source targetTemperature as Temperature;
     * }
     * </pre>
     */
    protected final static class ThermostatCompositeForStateFromContactSensor extends fr.inria.diagen.core.service.composite.Composite<ThermostatProxyForStateFromContactSensor> {
        private ThermostatCompositeForStateFromContactSensor(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Appliance/Thermostat/");
        }
        
        @Override
        protected ThermostatProxyForStateFromContactSensor instantiateProxy(RemoteServiceInfo rsi) {
            return new ThermostatProxyForStateFromContactSensor(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link ThermostatCompositeForStateFromContactSensor}, filtered using the attribute <code>id</code>.
         */
        public ThermostatCompositeForStateFromContactSensor andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link ThermostatCompositeForStateFromContactSensor}, filtered using the attribute <code>location</code>.
         */
        public ThermostatCompositeForStateFromContactSensor andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link ThermostatCompositeForStateFromContactSensor}, filtered using the attribute <code>user</code>.
         */
        public ThermostatCompositeForStateFromContactSensor andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
    }
    
    /**
     * A proxy to one <code>Thermostat</code> device to get its sources for the
     * <code>when provided state from ContactSensor</code> interaction contract.
     * 
     * <pre>
     * device Thermostat extends Appliance {
     * 	action SetTemperature;
     * 	source targetTemperature as Temperature;
     * }
     * </pre>
     */
    protected final static class ThermostatProxyForStateFromContactSensor extends Proxy {
        private ThermostatProxyForStateFromContactSensor(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Returns the value of the <code>status</code> source of this <code>Thermostat</code> device
         * 
         * @return the value of the <code>status</code> source
         */
        public java.lang.Boolean getStatus() throws InvocationException {
            return (java.lang.Boolean) callGetValue("status");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover object for state from ContactSensor
}
