package fr.inria.phoenix.diasuite.framework.controller.cutrestart;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.context.window.WindowValue;

/**
 * Cut/restart heating when window is open/closed
 * 
 * <pre>
 * controller CutRestart {
 * 	when provided Window do On on Thermostat, Off on Thermostat;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractCutRestart extends Service {
    
    public AbstractCutRestart(ServiceConfiguration serviceConfiguration) {
        super("/Controller/CutRestart/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("window", "/Context/Window/"); // subscribe to Window context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("window") && source.isCompatible("/Context/Window/")) {
            WindowValue windowValue = new WindowValue((java.lang.Boolean) value,
                    (java.lang.String) indexes[0]);
            
            onWindow(windowValue, new DiscoverForWindow());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>Window</code> context publishes a value.
     * 
     * <pre>
     * when provided Window do On on Thermostat, Off on Thermostat;
     * </pre>
     * 
     * @param window the value of the <code>Window</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onWindow(WindowValue window, DiscoverForWindow discover);
    
    // End of interaction contract implementation
    
    // Discover object for Window
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided Window do On on Thermostat, Off on Thermostat;
     * </code>
     */
    protected final class DiscoverForWindow {
        private final ThermostatDiscovererForWindow thermostatDiscoverer = new ThermostatDiscovererForWindow(AbstractCutRestart.this);
        
        /**
         * @return a {@link ThermostatDiscovererForWindow} object to discover <code>Thermostat</code> devices
         */
        public ThermostatDiscovererForWindow thermostats() {
            return thermostatDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>Thermostat</code> devices to execute action on for the
     * <code>when provided Window</code> interaction contract.
     * 
     * <pre>
     * device Thermostat extends Appliance {
     * 	action SetTemperature;
     * 	source targetTemperature as Temperature;
     * }
     * </pre>
     */
    protected final static class ThermostatDiscovererForWindow {
        private Service serviceParent;
        
        private ThermostatDiscovererForWindow(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private ThermostatCompositeForWindow instantiateComposite() {
            return new ThermostatCompositeForWindow(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices
         * 
         * @return a {@link ThermostatCompositeForWindow} object composed of all discoverable <code>Thermostat</code>
         */
        public ThermostatCompositeForWindow all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Thermostat</code> devices
         * 
         * @return a {@link ThermostatProxyForWindow} object pointing to a random discoverable <code>Thermostat</code> device
         */
        public ThermostatProxyForWindow anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link ThermostatCompositeForWindow} object composed of all matching <code>Thermostat</code> devices
         */
        public ThermostatCompositeForWindow whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link ThermostatCompositeForWindow} object composed of all matching <code>Thermostat</code> devices
         */
        public ThermostatCompositeForWindow whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>Thermostat</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link ThermostatCompositeForWindow} object composed of all matching <code>Thermostat</code> devices
         */
        public ThermostatCompositeForWindow whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>Thermostat</code> devices to execute action on for the
     * <code>when provided Window</code> interaction contract.
     * 
     * <pre>
     * device Thermostat extends Appliance {
     * 	action SetTemperature;
     * 	source targetTemperature as Temperature;
     * }
     * </pre>
     */
    protected final static class ThermostatCompositeForWindow extends fr.inria.diagen.core.service.composite.Composite<ThermostatProxyForWindow> {
        private ThermostatCompositeForWindow(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Appliance/Thermostat/");
        }
        
        @Override
        protected ThermostatProxyForWindow instantiateProxy(RemoteServiceInfo rsi) {
            return new ThermostatProxyForWindow(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link ThermostatCompositeForWindow}, filtered using the attribute <code>id</code>.
         */
        public ThermostatCompositeForWindow andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link ThermostatCompositeForWindow}, filtered using the attribute <code>location</code>.
         */
        public ThermostatCompositeForWindow andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link ThermostatCompositeForWindow}, filtered using the attribute <code>user</code>.
         */
        public ThermostatCompositeForWindow andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Executes the <code>on()</code> action's method on all devices of this composite.
         */
        public void on() throws InvocationException {
            launchDiscovering();
            for (ThermostatProxyForWindow proxy : proxies) {
                proxy.on();
            }
        }
        
        /**
         * Executes the <code>off()</code> action's method on all devices of this composite.
         */
        public void off() throws InvocationException {
            launchDiscovering();
            for (ThermostatProxyForWindow proxy : proxies) {
                proxy.off();
            }
        }
    }
    
    /**
     * A proxy to one <code>Thermostat</code> device to execute action on for the
     * <code>when provided Window</code> interaction contract.
     * 
     * <pre>
     * device Thermostat extends Appliance {
     * 	action SetTemperature;
     * 	source targetTemperature as Temperature;
     * }
     * </pre>
     */
    protected final static class ThermostatProxyForWindow extends Proxy {
        private ThermostatProxyForWindow(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Executes the <code>on()</code> action's method on this device.
         */
        public void on() throws InvocationException {
            callOrder("on");
        }
        
        /**
         * Executes the <code>off()</code> action's method on this device.
         */
        public void off() throws InvocationException {
            callOrder("off");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover object for Window
}
