package fr.inria.phoenix.diasuite.framework.controller.log;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.context.format.FormatValue;

/**
 * * HeatingMonitor controllers
 * Record events in the log
 * 
 * <pre>
 * controller Log {
 * 	when provided Format do Log on Logger;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractLog extends Service {
    
    public AbstractLog(ServiceConfiguration serviceConfiguration) {
        super("/Controller/Log/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("format", "/Context/Format/"); // subscribe to Format context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("format") && source.isCompatible("/Context/Format/")) {
            FormatValue formatValue = new FormatValue((fr.inria.phoenix.diasuite.framework.datatype.event.Event) value);
            
            onFormat(formatValue, new DiscoverForFormat());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>Format</code> context publishes a value.
     * 
     * <pre>
     * when provided Format do Log on Logger;
     * </pre>
     * 
     * @param format the value of the <code>Format</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onFormat(FormatValue format, DiscoverForFormat discover);
    
    // End of interaction contract implementation
    
    // Discover object for Format
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided Format do Log on Logger;
     * </code>
     */
    protected final class DiscoverForFormat {
        private final LoggerDiscovererForFormat loggerDiscoverer = new LoggerDiscovererForFormat(AbstractLog.this);
        
        /**
         * @return a {@link LoggerDiscovererForFormat} object to discover <code>Logger</code> devices
         */
        public LoggerDiscovererForFormat loggers() {
            return loggerDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>Logger</code> devices to execute action on for the
     * <code>when provided Format</code> interaction contract.
     * 
     * <pre>
     * device Logger extends Service {
     * 	action Log;
     * }
     * </pre>
     */
    protected final static class LoggerDiscovererForFormat {
        private Service serviceParent;
        
        private LoggerDiscovererForFormat(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private LoggerCompositeForFormat instantiateComposite() {
            return new LoggerCompositeForFormat(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Logger</code> devices
         * 
         * @return a {@link LoggerCompositeForFormat} object composed of all discoverable <code>Logger</code>
         */
        public LoggerCompositeForFormat all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Logger</code> devices
         * 
         * @return a {@link LoggerProxyForFormat} object pointing to a random discoverable <code>Logger</code> device
         */
        public LoggerProxyForFormat anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Logger</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link LoggerCompositeForFormat} object composed of all matching <code>Logger</code> devices
         */
        public LoggerCompositeForFormat whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
    }
    
    /**
     * A composite of several <code>Logger</code> devices to execute action on for the
     * <code>when provided Format</code> interaction contract.
     * 
     * <pre>
     * device Logger extends Service {
     * 	action Log;
     * }
     * </pre>
     */
    protected final static class LoggerCompositeForFormat extends fr.inria.diagen.core.service.composite.Composite<LoggerProxyForFormat> {
        private LoggerCompositeForFormat(Service serviceParent) {
            super(serviceParent, "/Device/Device/Service/Logger/");
        }
        
        @Override
        protected LoggerProxyForFormat instantiateProxy(RemoteServiceInfo rsi) {
            return new LoggerProxyForFormat(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link LoggerCompositeForFormat}, filtered using the attribute <code>id</code>.
         */
        public LoggerCompositeForFormat andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Executes the <code>logEvent(event as Event)</code> action's method on all devices of this composite.
         * 
         * @param event the <code>event</code> parameter of the <code>logEvent(event as Event)</code> method.
         */
        public void logEvent(fr.inria.phoenix.diasuite.framework.datatype.event.Event event) throws InvocationException {
            launchDiscovering();
            for (LoggerProxyForFormat proxy : proxies) {
                proxy.logEvent(event);
            }
        }
    }
    
    /**
     * A proxy to one <code>Logger</code> device to execute action on for the
     * <code>when provided Format</code> interaction contract.
     * 
     * <pre>
     * device Logger extends Service {
     * 	action Log;
     * }
     * </pre>
     */
    protected final static class LoggerProxyForFormat extends Proxy {
        private LoggerProxyForFormat(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Executes the <code>logEvent(event as Event)</code> action's method on this device.
         * 
         * @param event the <code>event</code> parameter of the <code>logEvent(event as Event)</code> method.
         */
        public void logEvent(fr.inria.phoenix.diasuite.framework.datatype.event.Event event) throws InvocationException {
            callOrder("logEvent", event);
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
    }
    // End of discover object for Format
}
