package fr.inria.phoenix.diasuite.framework.controller.mail;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.context.analysis.AnalysisValue;

/**
 * Send message about users needing an audit
 * 
 * <pre>
 * controller Mail {
 * 	when provided Analysis do SendMessage on Mailer; 
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractMail extends Service {
    
    public AbstractMail(ServiceConfiguration serviceConfiguration) {
        super("/Controller/Mail/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("analysis", "/Context/Analysis/"); // subscribe to Analysis context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("analysis") && source.isCompatible("/Context/Analysis/")) {
            AnalysisValue analysisValue = new AnalysisValue((java.lang.String) value);
            
            onAnalysis(analysisValue, new DiscoverForAnalysis());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>Analysis</code> context publishes a value.
     * 
     * <pre>
     * when provided Analysis do SendMessage on Mailer;
     * </pre>
     * 
     * @param analysis the value of the <code>Analysis</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onAnalysis(AnalysisValue analysis, DiscoverForAnalysis discover);
    
    // End of interaction contract implementation
    
    // Discover object for Analysis
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided Analysis do SendMessage on Mailer;
     * </code>
     */
    protected final class DiscoverForAnalysis {
        private final MailerDiscovererForAnalysis mailerDiscoverer = new MailerDiscovererForAnalysis(AbstractMail.this);
        
        /**
         * @return a {@link MailerDiscovererForAnalysis} object to discover <code>Mailer</code> devices
         */
        public MailerDiscovererForAnalysis mailers() {
            return mailerDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>Mailer</code> devices to execute action on for the
     * <code>when provided Analysis</code> interaction contract.
     * 
     * <pre>
     * device Mailer extends CommunicationService {
     * }
     * </pre>
     */
    protected final static class MailerDiscovererForAnalysis {
        private Service serviceParent;
        
        private MailerDiscovererForAnalysis(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private MailerCompositeForAnalysis instantiateComposite() {
            return new MailerCompositeForAnalysis(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Mailer</code> devices
         * 
         * @return a {@link MailerCompositeForAnalysis} object composed of all discoverable <code>Mailer</code>
         */
        public MailerCompositeForAnalysis all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Mailer</code> devices
         * 
         * @return a {@link MailerProxyForAnalysis} object pointing to a random discoverable <code>Mailer</code> device
         */
        public MailerProxyForAnalysis anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Mailer</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link MailerCompositeForAnalysis} object composed of all matching <code>Mailer</code> devices
         */
        public MailerCompositeForAnalysis whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
    }
    
    /**
     * A composite of several <code>Mailer</code> devices to execute action on for the
     * <code>when provided Analysis</code> interaction contract.
     * 
     * <pre>
     * device Mailer extends CommunicationService {
     * }
     * </pre>
     */
    protected final static class MailerCompositeForAnalysis extends fr.inria.diagen.core.service.composite.Composite<MailerProxyForAnalysis> {
        private MailerCompositeForAnalysis(Service serviceParent) {
            super(serviceParent, "/Device/Device/Service/CommunicationService/Mailer/");
        }
        
        @Override
        protected MailerProxyForAnalysis instantiateProxy(RemoteServiceInfo rsi) {
            return new MailerProxyForAnalysis(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link MailerCompositeForAnalysis}, filtered using the attribute <code>id</code>.
         */
        public MailerCompositeForAnalysis andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Executes the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> action's method on all devices of this composite.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param attachments the <code>attachments</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         */
        public void sendMessage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.util.List<fr.inria.phoenix.diasuite.framework.datatype.file.File> attachments) throws InvocationException {
            launchDiscovering();
            for (MailerProxyForAnalysis proxy : proxies) {
                proxy.sendMessage(to, title, content, attachments);
            }
        }
        
        /**
         * Executes the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> action's method on all devices of this composite.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param image the <code>image</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         */
        public void sendMessageWithImage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.lang.String image) throws InvocationException {
            launchDiscovering();
            for (MailerProxyForAnalysis proxy : proxies) {
                proxy.sendMessageWithImage(to, title, content, image);
            }
        }
    }
    
    /**
     * A proxy to one <code>Mailer</code> device to execute action on for the
     * <code>when provided Analysis</code> interaction contract.
     * 
     * <pre>
     * device Mailer extends CommunicationService {
     * }
     * </pre>
     */
    protected final static class MailerProxyForAnalysis extends Proxy {
        private MailerProxyForAnalysis(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Executes the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> action's method on this device.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param attachments the <code>attachments</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         */
        public void sendMessage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.util.List<fr.inria.phoenix.diasuite.framework.datatype.file.File> attachments) throws InvocationException {
            callOrder("sendMessage", to, title, content, attachments);
        }
        
        /**
         * Executes the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> action's method on this device.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param image the <code>image</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         */
        public void sendMessageWithImage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.lang.String image) throws InvocationException {
            callOrder("sendMessageWithImage", to, title, content, image);
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
    }
    // End of discover object for Analysis
}
