package fr.inria.phoenix.diasuite.framework.misc;

import fr.inria.diagen.core.deploy.AbstractDeploy;

import fr.inria.phoenix.diasuite.framework.context.analysis.AbstractAnalysis;
import fr.inria.phoenix.diasuite.framework.context.averageefficiency.AbstractAverageEfficiency;
import fr.inria.phoenix.diasuite.framework.context.efficiency.AbstractEfficiency;
import fr.inria.phoenix.diasuite.framework.context.format.AbstractFormat;
import fr.inria.phoenix.diasuite.framework.context.referenceefficiency.AbstractReferenceEfficiency;
import fr.inria.phoenix.diasuite.framework.context.window.AbstractWindow;

import fr.inria.phoenix.diasuite.framework.controller.cutrestart.AbstractCutRestart;
import fr.inria.phoenix.diasuite.framework.controller.log.AbstractLog;
import fr.inria.phoenix.diasuite.framework.controller.mail.AbstractMail;

/**
 * This class should be implemented to bind the implementation of the various components
 */
public abstract class AppComponentBinder extends AbstractDeploy {

    // Context instances
    private AbstractAnalysis analysisInstance = null;
    private AbstractAverageEfficiency averageEfficiencyInstance = null;
    private AbstractEfficiency efficiencyInstance = null;
    private AbstractFormat formatInstance = null;
    private AbstractReferenceEfficiency referenceEfficiencyInstance = null;
    private AbstractWindow windowInstance = null;

    // Controller instances
    private AbstractCutRestart cutRestartInstance = null;
    private AbstractLog logInstance = null;
    private AbstractMail mailInstance = null;
    
    @Override
    public void deployAll() {
        // Initialization of contexts
        if (analysisInstance == null)
            analysisInstance = getInstance(getAnalysisClass());
        if (averageEfficiencyInstance == null)
            averageEfficiencyInstance = getInstance(getAverageEfficiencyClass());
        if (efficiencyInstance == null)
            efficiencyInstance = getInstance(getEfficiencyClass());
        if (formatInstance == null)
            formatInstance = getInstance(getFormatClass());
        if (referenceEfficiencyInstance == null)
            referenceEfficiencyInstance = getInstance(getReferenceEfficiencyClass());
        if (windowInstance == null)
            windowInstance = getInstance(getWindowClass());
        // Intialization of controllers
        if (cutRestartInstance == null)
            cutRestartInstance = getInstance(getCutRestartClass());
        if (logInstance == null)
            logInstance = getInstance(getLogClass());
        if (mailInstance == null)
            mailInstance = getInstance(getMailClass());
        // Deploying contexts
        deploy(analysisInstance);
        deploy(averageEfficiencyInstance);
        deploy(efficiencyInstance);
        deploy(formatInstance);
        deploy(referenceEfficiencyInstance);
        deploy(windowInstance);
        // Deploying controllers
        deploy(cutRestartInstance);
        deploy(logInstance);
        deploy(mailInstance);
    }
    
    @Override
    public void undeployAll() {
        // Undeploying contexts
        undeploy(analysisInstance);
        undeploy(averageEfficiencyInstance);
        undeploy(efficiencyInstance);
        undeploy(formatInstance);
        undeploy(referenceEfficiencyInstance);
        undeploy(windowInstance);
        // Undeploying controllers
        undeploy(cutRestartInstance);
        undeploy(logInstance);
        undeploy(mailInstance);
    }
    
    // Abstract binding methods for contexts
    /**
     * Overrides this method to provide the implementation class of the <code>Analysis</code> context
     * <p>
     * Comparatively analyze efficiencies and detect users needing an audit
     * 
     * <pre>
     * context Analysis as String {
     * 	when provided tickHour from Clock
     * 		get AverageEfficiency, ReferenceEfficiency
     * 		maybe publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractAnalysis} that implements the <code>Analysis</code> context
     */
    public abstract Class<? extends AbstractAnalysis> getAnalysisClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>AverageEfficiency</code> context
     * <p>
     * Compute the average efficiency for each user
     * 
     * <pre>
     * context AverageEfficiency as EfficiencySample[] {
     * 	when provided Efficiency no publish;
     * 	when required;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractAverageEfficiency} that implements the <code>AverageEfficiency</code> context
     */
    public abstract Class<? extends AbstractAverageEfficiency> getAverageEfficiencyClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Efficiency</code> context
     * <p>
     * Compute instant efficiency for each heating measurement
     * 
     * <pre>
     * context Efficiency as EfficiencySample {
     * 	when provided currentHeatingConsumption from HeatingMeter
     * 		get targetTemperature from Thermostat,
     * 			temperature from TemperatureSensor
     * 		always publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractEfficiency} that implements the <code>Efficiency</code> context
     */
    public abstract Class<? extends AbstractEfficiency> getEfficiencyClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Format</code> context
     * <p>
     * * HeatingMonitor contexts
     * Format various events for logging purpose
     * 
     * <pre>
     * context Format as Event {
     * 	when provided currentHeatingConsumption from HeatingMeter
     * 		always publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractFormat} that implements the <code>Format</code> context
     */
    public abstract Class<? extends AbstractFormat> getFormatClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>ReferenceEfficiency</code> context
     * <p>
     * Compute the reference efficiency as the average of all users
     * 
     * <pre>
     * context ReferenceEfficiency as Float {
     * 	when provided Efficiency no publish;
     * 	when required;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractReferenceEfficiency} that implements the <code>ReferenceEfficiency</code> context
     */
    public abstract Class<? extends AbstractReferenceEfficiency> getReferenceEfficiencyClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Window</code> context
     * <p>
     * Cut/restart heating when window is open/closed
     * 
     * <pre>
     * context Window as Boolean indexed by user as String {
     * 	when provided state from ContactSensor
     * 	get status from Thermostat maybe publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractWindow} that implements the <code>Window</code> context
     */
    public abstract Class<? extends AbstractWindow> getWindowClass();
    
    // End of abstract binding methods for contexts
    
    // Abstract binding methods for controllers
    /**
     * Overrides this method to provide the implementation class of the <code>CutRestart</code> controller
     * <p>
     * Cut/restart heating when window is open/closed
     * 
     * <pre>
     * controller CutRestart {
     * 	when provided Window do On on Thermostat, Off on Thermostat;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractCutRestart} that implements the <code>CutRestart</code> controller
     */
    public abstract Class<? extends AbstractCutRestart> getCutRestartClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Log</code> controller
     * <p>
     * * HeatingMonitor controllers
     * Record events in the log
     * 
     * <pre>
     * controller Log {
     * 	when provided Format do Log on Logger;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractLog} that implements the <code>Log</code> controller
     */
    public abstract Class<? extends AbstractLog> getLogClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Mail</code> controller
     * <p>
     * Send message about users needing an audit
     * 
     * <pre>
     * controller Mail {
     * 	when provided Analysis do SendMessage on Mailer; 
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractMail} that implements the <code>Mail</code> controller
     */
    public abstract Class<? extends AbstractMail> getMailClass();
    
    // End of abstract binding methods for controllers
}
